/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "defines.h"
#include "Parser.h"

Parser::Parser()
{
    
}

void Parser::addToken(const QString &tokenName, const QString &tokenValue)
{
    tokens[tokenName] = tokenValue;
}

void Parser::setTokenValue(const QString &tokenName, const QString &tokenValue)
{
    tokens[tokenName] = tokenValue;
}

const QString& Parser::getTokenValue(QString &tokenName) const
{
    TokenList::const_iterator iter = tokens.find(tokenName);
    
    return iter.value();
}

QString Parser::parse(const QString &strToParse)
{

    int pos1, pos2;
    QString start, end, elem;
    QString buffer = strToParse;

    for(TokenList::iterator iter = tokens.begin(); iter != tokens.end(); ++iter)
    {
        start   = BLOCK_START_L+iter.key()+BLOCK_START_R;
        end     = BLOCK_END_L+iter.key()+BLOCK_END_R;
        elem    = ELEMENT_L+iter.key()+ELEMENT_R;

        // find block elements and erase accordingly
        pos1 = buffer.indexOf(start, 0, Qt::CaseSensitive);
        while(pos1 > 0 ){
            pos2 = buffer.indexOf(end, pos1, Qt::CaseSensitive);

            if(pos2 > 0){
                if( iter.value().isEmpty() ){
                    buffer.remove(pos1, pos2-pos1+end.size() );
                }
                else
                {
                    buffer.remove(pos2, end.size() );
                    buffer.remove(pos1, start.size() );
                }
            }
            pos1 = buffer.indexOf(start);
        }

        // replace all token elements found in buffer with their value
        pos1 = buffer.indexOf(elem);
        pos2 = elem.size();
        while(pos1 > 0){
            buffer.replace(pos1, pos2, iter.value() );
            pos1 = buffer.indexOf(elem);
        }


    }

   return buffer;
}
